/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Test;
import org.mockito.Mockito;
import org.ow2.petals.commons.log.FlowAttributes;
import org.ow2.petals.commons.log.PetalsExecutionContext;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class ContextualMessageExchangeImplBuilderTest extends MessageExchangeCheckHelper {

    private ServiceEndpoint consumerEndpointMock = Mockito.mock(ServiceEndpoint.class);

    @After
    public void after() {
        ExecutionContext.getProperties().clear();
    }

    @Test
    public void testBuildUseBuilderConsumerEp() throws MessagingException {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);
        MessageExchangeImpl messageExchangeImpl = messageExchangeImplBuilder.build();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForEndpoint() throws MessagingException {
        ServiceEndpoint targetEndpointMock = Mockito.mock(ServiceEndpoint.class);
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilderForEndpoint(this.consumerEndpointMock, targetEndpointMock);
        MessageExchangeImpl messageExchangeImpl = messageExchangeImplBuilder.build();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetEndpointMock, messageExchangeImpl.getEndpoint());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForInterface() throws MessagingException {
        QName targetInterface = new QName("http://test","interface");
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilderForInterface(this.consumerEndpointMock, targetInterface);
        MessageExchangeImpl messageExchangeImpl = messageExchangeImplBuilder.build();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetInterface, messageExchangeImpl.getInterfaceName());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForService() throws MessagingException {
        QName targetService = new QName("http://test","service");
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilderForService(this.consumerEndpointMock, targetService);
        MessageExchangeImpl messageExchangeImpl = messageExchangeImplBuilder.build();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetService, messageExchangeImpl.getService());
    }

    @Test
    public void testBuildGenerateUUIDExchangeId() throws MessagingException {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);
        MessageExchangeImpl messageExchangeImpl = messageExchangeImplBuilder.build();

        String createdExchangeId = messageExchangeImpl.getExchangeId();
        assertIdIsPetalsQualifiedUUID(createdExchangeId);
    }

    @Test
    public void testBuildWithNonNullFlowAttributesInContext() throws MessagingException {
        FlowAttributes flowAttributes = new FlowAttributes("flowInstanceId1", "flowStepId1");
        PetalsExecutionContext.putFlowAttributes(flowAttributes);

        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);
        MessageExchangeImpl actualMessageExchangeImpl = messageExchangeImplBuilder.build();
        assertEquals(flowAttributes, actualMessageExchangeImpl.getFlowAttributes());
    }

    @Test
    public void testBuildWithNoFlowAttributesInContext() throws MessagingException {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);
        MessageExchangeImpl actualMessageExchangeImpl = messageExchangeImplBuilder.build();
        assertNull(actualMessageExchangeImpl.getFlowAttributes());
    }

    @Test
    public void testBuildButMissingFlowInstanceIdInContext() throws MessagingException {
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, "flowStepId1");
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);        
        MessageExchangeImpl actualMessageExchangeImpl = messageExchangeImplBuilder.build();
        assertNull(actualMessageExchangeImpl.getFlowAttributes());
    }

    @Test
    public void testBuildButMissingFlowStepIdInContext() throws MessagingException {
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, "flowInstanceId1");
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder.createMessageExchangeImplBuilder(this.consumerEndpointMock);        
        MessageExchangeImpl actualMessageExchangeImpl = messageExchangeImplBuilder.build();
        assertNull(actualMessageExchangeImpl.getFlowAttributes());
    }

}
